/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) 2001 Bruce Harris
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere;

import javax.swing.tree.DefaultMutableTreeNode;
import belvedere.model.Model;
import belvedere.model.Constituent;

/**
 * Customized node type using in JTreePanel
 */
public class BelMutableTreeNode extends DefaultMutableTreeNode {

  /**
   * Basic constructor
   */
  public BelMutableTreeNode() {
    this(null);
  } 

  /**
   * Constructor that sets user object
   * 
   * @param userObject object to be held by the MutableTreeNode
   */
  public BelMutableTreeNode(Object userObject) {
    this(userObject, true);
  }

  /**
   * Constructor that sets user object and also flags whether or not to allow children
   * 
   * @param userObject object to be held by the MutableTreeNode
   * @param allowsChildren
   *                   if true node can have children, otherwise no children
   */
  public BelMutableTreeNode(Object userObject, boolean allowsChildren) {
    super(userObject, allowsChildren);
  } 

  /**
   * Overwrites the standard toString method to return the
   * appropriate text associated with the contained object
   *
   * @return <code>String</code> representation of the object 
   *         contained within this node or null if there is
   *         no such object.
   */
  public String toString() {
    if (userObject == null) {
      return null;
    } else if (userObject instanceof Model) {
      return((Model)userObject).getName();
    } else if (userObject instanceof Constituent) {
      String text = ((Constituent)userObject).getText();
      if (text.trim().equals(""))
        return "---";
      else
        return text;   
    } else {
      return userObject.toString();
    }
  }
}
