/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) <year> <name of author>
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.ResourceBundle;
import java.util.Locale;
import java.util.ArrayList;

import java.lang.reflect.*;

import belvedere.model.*;
import belvedere.resources.*;
import belvedere.io.*;
import belvedere.util.ObjectFactory;
import belvedere.util.DragDropManager;
import belvedere.util.DropTarget;
import belvedere.util.CursorFactory;

import java.util.Iterator;

import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.IOException;

import belvedere.graphview.VisualNode;
import belvedere.model.ModelChangeListener;
import belvedere.model.ProjectChangeListener;
import belvedere.io.Belvedere3Importer;

import org.jdom.JDOMException;

public class Belvedere4 extends JFrame implements DragDropManager, ModelChangeListener, ProjectChangeListener {

  Project prj = null;

  String filename = null;

  private Cursor dragCursor;

  // Panels
  JTabbedPane pnlTop;
  JTabbedPane pnlBottom;
  JPanel panel2;
  ProjectTreePanel pnlLeft;
  BelvedereGlassPane glassPane;

  // Main menubar
  JMenuBar menuMain;
  // "File" menu
  JMenuItem menuitemNew;
  JMenuItem menuitemOpen;
  JMenuItem menuitemSave;
  JMenuItem menuitemSaveAs;
  JMenuItem menuitemExit;
  // "Project" menu
  JMenuItem menuitemAddModel;
  JMenuItem menuitemImportModel;
  JMenuItem menuitemProjectProperties;
  JMenuItem menuitemRemoveModel;
  // "Help" menu
  JMenuItem menuitemAbout;
  // Main toolbar
  JToolBar tbMain;
  JButton buttonNew;
  JButton buttonOpen;
  JButton buttonSave;
  JButton buttonAbout;
  // Open and Save file dialogs
  FileDialog openFileDialog;
  FileDialog saveFileDialog;
  // Images
  ImageIcon iconNew;
  ImageIcon iconOpen;
  ImageIcon iconSave;
  ImageIcon iconAbout;


  private static final String DEFAULT_PROPERTIES = "belvedere.resources.BelvedereProperties";
  private static ResourceBundle properties;  // All static text and text references

  public Belvedere4(String title) {

    // Initialize
    //
    // Common images for menubar and toolbar
    LoadCommonImages();

    //set up drag and drop cursors for the application
    dragCursor = CursorFactory.getCursor(this.properties.getString("DRAG_CURSOR"),
                                         "drag cursor",new Point(0,0),this);

    // Frame
    setTitle(title);
    setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

    setSize(600,600);
    setVisible(false);
    // Menu
    menuMain = new JMenuBar();
    setJMenuBar(menuMain);
    menuAddItems(menuMain);

    // Panels
    getContentPane().setLayout(new BorderLayout(0,0));

    pnlTop = new JTabbedPane();
    pnlBottom = new JTabbedPane();
    JSplitPane pnlRight = new JSplitPane(JSplitPane.VERTICAL_SPLIT,pnlTop,pnlBottom);
    pnlRight.setOneTouchExpandable(true);
    pnlRight.setDividerLocation(0.0);

    pnlLeft = new ProjectTreePanel(this);

    JSplitPane split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,pnlLeft,pnlRight);

    split.setOneTouchExpandable(true);
    split.setDividerLocation(200);
    getContentPane().add(BorderLayout.CENTER,split);


    panel2 = new JPanel(new FlowLayout(FlowLayout.LEFT,0,0));
    getContentPane().add(BorderLayout.NORTH, panel2);
    panel2.setBounds(0,0,475,30);
    // Toolbar
    tbMain = new JToolBar();
    tbMain.setFloatable(false);
    panel2.add(tbMain);
    tbMain.setBounds(0,0,375,30);
    tbAddButtons(tbMain);

    // set up the GlassPane
    glassPane = new BelvedereGlassPane();
    this.setGlassPane(glassPane);

    // Open and Save file dialogs
    openFileDialog = new FileDialog(this, properties.getString("OPEN"),
                                    FileDialog.LOAD);

    // add filters here



    openFileDialog.setModal(true);
    saveFileDialog = new FileDialog(this, properties.getString("SAVE"),
                                    FileDialog.SAVE);
    saveFileDialog.setModal(true);

    // Register listeners
    //
    // Window listener
    EvtListener elWindow = new EvtListener();
    this.addWindowListener(elWindow);
    // Menu and toolbar listener
    ActListener alMenuToolbar = new ActListener();
    menuitemNew.addActionListener(alMenuToolbar);
    menuitemOpen.addActionListener(alMenuToolbar);
    menuitemSave.addActionListener(alMenuToolbar);
    menuitemSaveAs.addActionListener(alMenuToolbar);
    menuitemExit.addActionListener(alMenuToolbar);
    menuitemAddModel.addActionListener(alMenuToolbar);
    menuitemImportModel.addActionListener(alMenuToolbar);
    menuitemProjectProperties.addActionListener(alMenuToolbar);
    menuitemRemoveModel.addActionListener(alMenuToolbar);
    //menuitemAbout.addActionListener(alMenuToolbar);
    buttonNew.addActionListener(alMenuToolbar);
    buttonOpen.addActionListener(alMenuToolbar);
    buttonSave.addActionListener(alMenuToolbar);
    //buttonAbout.addActionListener(alMenuToolbar);
    this.setVisible(true);
    //open interactive window for opening application
    new OpenApplicationFrame(this, properties.getString("WELCOME"));
  }

  /**
   * Create a new instance of Belvedere4 with default title.
   * @see #Belvedere4(String title)
   */
  public Belvedere4() {
    this("Belvedere 4.0");
  }


  public static ResourceBundle getResources() {
    return properties;
  }

  protected void LoadCommonImages()
  {
    iconNew   = new ImageIcon(properties.getString("ICON_NEW"));
    iconOpen  = new ImageIcon(properties.getString("ICON_OPEN"));
    iconSave  = new ImageIcon(properties.getString("ICON_SAVE"));
    iconAbout = new ImageIcon(properties.getString("ICON_ABOUT"));
  }

  protected void tbAddButtons(JToolBar toolbar) {
    Rectangle bounds = new Rectangle();
    // Bounds for each button
    bounds.x=2;
    bounds.y=2;
    bounds.width=25;
    bounds.height=25;
    // Toolbar separator
    Dimension separator = new Dimension(5,5);
    // Button size
    Dimension buttonsize = new Dimension(bounds.width,bounds.height);

    // New
    buttonNew = new JButton(iconNew);
    buttonNew.setDefaultCapable(false);
    buttonNew.setToolTipText(properties.getString("NEW_TIP"));
    buttonNew.setMnemonic((int)'N');
    toolbar.add(buttonNew);
    buttonNew.setBounds(bounds);
    buttonNew.setMinimumSize(buttonsize);
    buttonNew.setMaximumSize(buttonsize);
    buttonNew.setPreferredSize(buttonsize);
    // Open
    buttonOpen = new JButton(iconOpen);
    buttonOpen.setDefaultCapable(false);
    buttonOpen.setToolTipText(properties.getString("OPEN_TIP"));
    buttonOpen.setMnemonic((int)'O');
    toolbar.add(buttonOpen);
    bounds.x += bounds.width;
    buttonOpen.setBounds(bounds);
    buttonOpen.setMinimumSize(buttonsize);
    buttonOpen.setMaximumSize(buttonsize);
    buttonOpen.setPreferredSize(buttonsize);
    // Save
    buttonSave = new JButton(iconSave);
    buttonSave.setDefaultCapable(false);
    buttonSave.setToolTipText(properties.getString("SAVE_TIP"));
    buttonSave.setMnemonic((int)'S');
    toolbar.add(buttonSave);
    bounds.x += bounds.width;
    buttonSave.setBounds(bounds);
    buttonSave.setMinimumSize(buttonsize);
    buttonSave.setMaximumSize(buttonsize);
    buttonSave.setPreferredSize(buttonsize);
    // Separator
    toolbar.addSeparator(separator);
    // Separator
    toolbar.addSeparator(separator);

    /* removed until fully implemented help.

    // About
    buttonAbout = new JButton(iconAbout);
    buttonAbout.setDefaultCapable(false);
    buttonAbout.setToolTipText(properties.getString("ABOUT_TIP"));
    buttonAbout.setMnemonic((int)'A');
    toolbar.add(buttonAbout);
    bounds.x += bounds.width;
    buttonAbout.setBounds(bounds);
    buttonAbout.setMinimumSize(buttonsize);
    buttonAbout.setMaximumSize(buttonsize);
    buttonAbout.setPreferredSize(buttonsize);
    */
  }

  protected void menuAddItems(JMenuBar hmenu) {
    JMenu menu;
    JMenuItem item;

    //
    // "File" menu
    //
    menu = new JMenu(properties.getString("FILE"));
    menu.setActionCommand(properties.getString("FILE"));
    menu.setBorderPainted(false);
    menu.setMnemonic((int)'F');
    hmenu.add(menu);
    // "New" item
    menuitemNew = new JMenuItem(properties.getString("NEW"));
    menuitemNew.setHorizontalTextPosition(SwingConstants.RIGHT);
    menuitemNew.setActionCommand(properties.getString("NEW"));
    menuitemNew.setBorderPainted(false);
    menuitemNew.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, Event.CTRL_MASK));
    menuitemNew.setMnemonic((int)'N');
    menuitemNew.setIcon(iconNew);
    menu.add(menuitemNew);
    // "Open" item
    menuitemOpen = new JMenuItem(properties.getString("OPEN"));
    menuitemOpen.setHorizontalTextPosition(SwingConstants.RIGHT);
    menuitemOpen.setActionCommand(properties.getString("OPEN"));
    menuitemOpen.setBorderPainted(false);
    menuitemOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, Event.CTRL_MASK));
    menuitemOpen.setMnemonic((int)'O');
    menuitemOpen.setIcon(iconOpen);
    menu.add(menuitemOpen);
    // "Save" item
    menuitemSave = new JMenuItem(properties.getString("SAVE"));
    menuitemSave.setHorizontalTextPosition(SwingConstants.RIGHT);
    menuitemSave.setActionCommand(properties.getString("SAVE"));
    menuitemSave.setBorderPainted(false);
    menuitemSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, Event.CTRL_MASK));
    menuitemSave.setMnemonic((int)'S');
    menuitemSave.setIcon(iconSave);
    menu.add(menuitemSave);
    // "Save As" item
    menuitemSaveAs = new JMenuItem(properties.getString("SAVE_AS"));
    menuitemSaveAs.setActionCommand(properties.getString("SAVE_AS"));
    menuitemSaveAs.setBorderPainted(false);
    menuitemSaveAs.setMnemonic((int)'A');
    menu.add(menuitemSaveAs);
    // Separator
    menu.add(new JSeparator());
    // "Exit" item
    menuitemExit = new JMenuItem(properties.getString("EXIT"));
    menuitemExit.setActionCommand(properties.getString("EXIT"));
    menuitemExit.setBorderPainted(false);
    menuitemExit.setMnemonic((int)'X');
    menu.add(menuitemExit);
    //
    // "Project" menu
    //
    menu = new JMenu(this.properties.getString("PROJECT"));
    menu.setActionCommand(this.properties.getString("PROJECT"));
    menu.setBorderPainted(false);
    menu.setMnemonic((int)'P');
    hmenu.add(menu);

    // "AddModel" item
    menuitemAddModel = new JMenuItem(this.properties.getString("ADD_MODEL"));
    menuitemAddModel.setHorizontalTextPosition(SwingConstants.RIGHT);
    menuitemAddModel.setActionCommand(this.properties.getString("ADD_MODEL"));
    menuitemAddModel.setBorderPainted(false);
    menuitemAddModel.setMnemonic((int)'A');
    menuitemAddModel.setIcon(iconNew);
    menu.add(menuitemAddModel);

    // "ImportModel" item
    menuitemImportModel = new JMenuItem(this.properties.getString("IMPORT_MODEL"));
    menu.add(menuitemImportModel);
    // "ProjectProperties" item
    menuitemProjectProperties = new JMenuItem(this.properties.getString("PROJECT_PROPERTIES"));
    menu.add(menuitemProjectProperties);
    // "RemoveModel" item
    menuitemRemoveModel = new JMenuItem(this.properties.getString("REMOVE_MODEL"));
    menu.add(menuitemRemoveModel);



    /*
     **  temporarily removed until help is actually implemented ****
    //
    // "Help" menu
    //
    menu = new JMenu(properties.getString("HELP"));
    menu.setActionCommand(properties.getString("HELP"));
    menu.setBorderPainted(false);
    menu.setMnemonic((int)'H');
    hmenu.add(menu);
    // "About" item
    menuitemAbout = new JMenuItem(properties.getString("ABOUT"));
    menuitemAbout.setHorizontalTextPosition(SwingConstants.RIGHT);
    menuitemAbout.setActionCommand(properties.getString("ABOUT"));
    menuitemAbout.setBorderPainted(false);
    menuitemAbout.setMnemonic((int)'A');
    menuitemAbout.setIcon(iconAbout);
    menu.add(menuitemAbout);
    ** end  help */
  }


  /**
   * The entry point for this application.
   */
  static public void main(String args[]) {
    try {
      // Set the "look and feel" to the native system
      //try {
      //  UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
      //} catch (Exception e) {
      //}
      // Create a new instance of the application frame and make it visible

      // Get the locale: default, or specified on command-line
      Locale locale;
      if (args.length == 2) locale = new Locale(args[0], args[1]);
      else locale = Locale.getDefault();

      properties = ResourceBundle.getBundle(DEFAULT_PROPERTIES,
                                            locale);

      (new Belvedere4("Belvedere 4.0 " +
                      locale.getDisplayName(Locale.getDefault()))).setVisible(true);
    } catch (Exception e) {
      System.err.println(e);
      e.printStackTrace();
      // Exit with error condition
      System.exit(1);
    }
  }

  // Used by addNotify
  boolean addNotify_done=false;

  /**
   * Makes this Container displayable by connecting it to
   * a native screen resource.  Making a container displayable will
   * cause any of its children to be made displayable.
   * This method is called internally by the toolkit and should
   * not be called directly by programs.
   * <p>
   * Overridden here to adjust the size of the frame if needed.
   * </p>
   * @see java.awt.Component#isDisplayable
   * @see java.awt.Container#removeNotify
   */
  public void addNotify() {
    Dimension d=getSize();

    super.addNotify();

    if (addNotify_done) return;

    // Adjust size according to the insets so that entire component
    // areas are renderable.
    int menubarheight=0;
    JMenuBar menubar = getRootPane().getJMenuBar();
    if (menubar!=null) {
      menubarheight = menubar.getPreferredSize().height;
    }
    Insets insets=getInsets();
    setSize(insets.left + insets.right + d.width, insets.top + insets.bottom + d.height + menubarheight);
    addNotify_done=true;
  }

  protected void AboutApplication() {
    try {
      JOptionPane.showMessageDialog(this,
                                    "Belvedere 4.0",
                                    properties.getString("ABOUT"),
                                    JOptionPane.INFORMATION_MESSAGE);
    } catch (Exception e) {
    }
  }

  protected void exitApplication() {
    // Show an Exit confirmation dialog if they have unsaved work

    if (prj!=null && prj.isDirty()) {
      int reply = JOptionPane.showConfirmDialog(this,
                                                properties.getString("LOSE_CHANGES_EXIT"),
                                                properties.getString("EXIT"),
                                                JOptionPane.YES_NO_OPTION,
                                                JOptionPane.QUESTION_MESSAGE);
      if (reply==JOptionPane.NO_OPTION) return;
    }

    // User answered "Yes", so cleanup and exit
    //
    // Hide the frame
    this.setVisible(false);
    // Free system resources
    this.dispose();
    // Exit the application
    System.exit(0);

  }

  class EvtListener extends WindowAdapter {
    public void windowClosing(WindowEvent event) {
      Object object = event.getSource();
      if (object==Belvedere4.this) {
        Belvedere4_windowClosing(event);
      }
    }
  }

  void Belvedere4_windowClosing(WindowEvent event) {
    this.exitApplication();
  }

  class ActListener implements ActionListener {
    public void actionPerformed(ActionEvent event) {
      Object object = event.getSource();
      if (object==menuitemNew) {
        new_Action(event);
      } else if (object==menuitemAddModel) {
        create_Model_Action(event);
      } else if (object==menuitemOpen) {
        open_Action(event);
      } else if (object==menuitemImportModel) {
        import_Action(event);
      } else if (object==menuitemProjectProperties) {
        properties_Action(event);
      } else if (object==menuitemRemoveModel) {
        remove_Action(event);
      } else if (object==menuitemSave) {
        save_Action(event);
      } else if (object==menuitemSaveAs) {
        saveAs_Action(event);
      } else if (object==menuitemExit) {
        exit_Action(event);
      } else if (object==menuitemAbout) {
        about_Action(event);
      } else if (object==buttonNew) {
        new_Action(event);
      } else if (object==buttonOpen) {
        open_Action(event);
      } else if (object==buttonSave) {
        save_Action(event);
      } else if (object==buttonAbout) {
        about_Action(event);
      }
    }
  }

  void new_Action(ActionEvent event) {
    // check to see if they have a dirty open project
    if (prj!=null && prj.isDirty()) {
      int reply = JOptionPane.showConfirmDialog(this,
                                                this.properties.getString ("LOSE_CHANGES_NEW"),
                                                this.properties.getString("CONTINUE"),
                                                JOptionPane.YES_NO_OPTION,
                                                JOptionPane.QUESTION_MESSAGE);

      if (reply==JOptionPane.NO_OPTION) return;
    }

    //set the filename to null so that they will be prompted for it when
    //they try to save
    filename = null;

    // close the current project if any
    pnlTop.removeAll();
    pnlBottom.removeAll();
    //need to repaint?
    prj = null;

    // create a new empty one
    prj = new Project("New Project");
    pnlLeft.setProject(prj);
    prj.addProjectChangeListener(this);
    prj.addProjectChangeListener(pnlLeft);
    create_Model_Action(event);
  }

  void addModel_Action(ActionEvent event) {
    // this can only be called if a project is open because otherwise the
    // button to call this is disabled
    String modelName = JOptionPane.showInputDialog(this,
                                                   this.properties.getString("MODEL_NAME"));
    if (modelName == null || modelName.trim().equals(""))
      return;
    Model mdl = (Model)ObjectFactory.createObject(event.getActionCommand());
    if (mdl!=null) {
      mdl.setName(modelName);
      prj.addModel(mdl);
    }  //else we should do something, but this will only happen if the
    //action command for the add model menu items doesn't match one
    //of the classes for models
  }

  void open_Action(ActionEvent event) {
    Project prjOpen = null;
    Model mdl;

    // check to see if they have a dirty open project

    if (prj!=null && prj.isDirty()) {
      int reply = JOptionPane.showConfirmDialog(this,
                                                this.properties.getString("LOSE_CHANGES_NEW"),
                                                this.properties.getString("CONTINUE"),
                                                JOptionPane.YES_NO_OPTION,
                                                JOptionPane.QUESTION_MESSAGE);
      if (reply==JOptionPane.NO_OPTION) return;
    }

    openFileDialog.setVisible(true);
    if (openFileDialog.getFile()!=null) {
      prj = new Project();
      try {
        switch (FileVersioner.getVersion(openFileDialog.getDirectory() + openFileDialog.getFile())) {
          case FileVersioner.VERSION_3:
            Belvedere3Importer b3 = new Belvedere3Importer();
            prjOpen = b3.readFromXML(openFileDialog.getDirectory() + openFileDialog.getFile());
            JOptionPane.showMessageDialog(this,
                                          this.properties.getString("VERSION_3_MSG"),
                                          this.properties.getString("VERSION_3_TITLE"),
                                          JOptionPane.WARNING_MESSAGE);
            break;
          case FileVersioner.VERSION_4:
            prjOpen = new Project();
            prjOpen.read(openFileDialog.getDirectory() + openFileDialog.getFile());
            break;
          case FileVersioner.INVALID_FILE:
            throw new Exception();
        }
        // since we opened successfully, now close the current project
        pnlTop.removeAll();
        pnlBottom.removeAll();
        prj = prjOpen;
        pnlLeft.setProject(prj);
        prj.addProjectChangeListener(this);
        prj.addProjectChangeListener(pnlLeft);
        prj.setDirty(false);        
        for (Iterator i=prj.getModels().iterator();i.hasNext();) {
          mdl = (Model)i.next();
          //here we fake the model added to get the ui elements added
          this.modelAdded(prj,mdl);
          pnlLeft.modelAdded(prj,mdl);
        }
        pnlLeft.forceReload();
        //now that we have successfully opened the file set the filename
        //for future saves
        filename = openFileDialog.getDirectory() + openFileDialog.getFile();
        // could catch IOException or JDOMException here for better
        // description of the problem
      } catch (Exception e) {
        JOptionPane.showMessageDialog(this,
                                      this.properties.getString("IO_ERROR_OPEN"),
                                      this.properties.getString("IO_ERROR"),
                                      JOptionPane.ERROR_MESSAGE);
      }
    }
  }


  /**
   *
   **/
  void create_Model_Action(ActionEvent event) {
    ModelCreationFrame modelFrame = new ModelCreationFrame(this,
                                       this.properties.getString("CREATE_NEW_MODEL"));
  }


  /**
   * Imports a picomap model into the current project
   **/
  void import_Action(ActionEvent event) {
    PicoMapImporter importer = new PicoMapImporter(this);
    importer.openPicoMapFile();
    Model mdl = importer.importModel();
    if (mdl != null) {
      prj.addModel(mdl);
      prj.setDirty(true);
    }
  }

  void properties_Action(ActionEvent event) {
    ProjectModelPropertyEditor p = new ProjectModelPropertyEditor(this,
                                               this.properties.getString("PROJECT_PROPERTIES"),prj);
    p.showEditor();
  }

  void remove_Action(ActionEvent event) {
    ModelRemover mr = new ModelRemover(this,this.properties.getString("REMOVE_MODEL"),prj);
    mr.showModelRemover();
  }

  void save_Action(ActionEvent event) {
    if (filename!=null)
      save(filename);
    else
      saveAs_Action(event);
  }

  void saveAs_Action(ActionEvent event) {
    ModelPanel mp;
    if (prj==null) return;

    // Show saveFileDialog modal
    saveFileDialog.setVisible(true);
    if (saveFileDialog.getFile()!=null) {
      save(saveFileDialog.getDirectory() + saveFileDialog.getFile());
      filename = saveFileDialog.getDirectory() + saveFileDialog.getFile();
    }
  }

  void save(String filename) {
    FileOutputStream fos = null;
    ObjectOutputStream out = null;
    try {
      prj.save(filename);
      prj.setDirty(false);
    } catch (IOException e) {
      JOptionPane.showMessageDialog(this,
                                    this.properties.getString("IO_ERROR_SAVE") +
                                    e.getMessage(),
                                    this.properties.getString("IO_ERROR"),
                                    JOptionPane.ERROR_MESSAGE);
    }
  }

  void exit_Action(ActionEvent event) {
    this.exitApplication();
  }

  void about_Action(ActionEvent event) {
    // TODO:  code goes here

    try {
      this.AboutApplication();
    } catch (Exception e) {
    }
  }

  /**
   * subcomponents of the application JFrame call this method in response to
   * mouseRelease events when in the dragging state.  This method forwards the
   * drop to the appropriate component
   *
   * @param x      x coordinate of drop
   * @param y      y coordinate of drop
   * @param o      Object associated with drop
   */
  public void forwardDrop(int x, int y, Object o) {
    // first check if dropped on pnlTop
    Point p = pnlTop.getLocationOnScreen();
    if (pnlTop.contains(x-(int)p.getX(),y-(int)p.getY())) {
      ((DropTarget)pnlTop.getSelectedComponent()).drop(x,y,o);
    }
    // now check pnlBottom drop
    p = pnlBottom.getLocationOnScreen();
    if (pnlBottom.contains(x-(int)p.getX(),y-(int)p.getY())) {
      ((DropTarget)pnlBottom.getSelectedComponent()).drop(x,y,o);
    }
    // change the mouse cursor back
    this.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    glassPane.setDragObject(null);
    glassPane.setVisible(false);
  }

  /**
   * subcomponents of the appliation JFrame call this method in response to
   * the start of the drag, useful for setting proper mouse cursor
   */
  public void startDrag(Object o, MouseEvent e) {
    this.setCursor(dragCursor);
    glassPane.setVisible(true);
    glassPane.setDragObject(o);
    // make sure the first drag event gets fired on the glass pane
    // so that the original drawing location is correct
    glassPane.dispatchEvent(SwingUtilities.convertMouseEvent(e.getComponent(),e,glassPane));
  }


  /**
   * Internal class for initializing the application upon opening.
   */
  class OpenApplicationFrame extends JDialog {

    // Panels
    private JPanel northPanel;
    private JPanel centerPanel;
    private JPanel mainPanel;
    private JPanel openProjPanel;
    private JPanel newProjPanel;
    private JPanel exitAppPanel;
    private JPanel openButtonPanel;
    private JPanel newButtonPanel;
    private JPanel exitButtonPanel;

    // Labels
    private JLabel belLabel;
    private JLabel dialogLabel;
    private JLabel newProjLabel;
    private JLabel openProjLabel;
    private JLabel spacer = new JLabel(" ");

    // Images
    //private ImageIcon belvIcon;
    private ImageIcon newProjIcon;
    private ImageIcon openProjIcon;

    // Buttons
    private JButton newProjButton;
    private JButton openProjButton;
    private JButton exitAppButton;

    // Listeners
    private OpenAppActListener buttonListener;

    private FileDialog openFileDialog;


    public OpenApplicationFrame(JFrame parent, String title) {
      super(parent, title, true);
      // setup Panels
      northPanel = new JPanel();
      centerPanel = new JPanel();
      mainPanel = new JPanel();
      openButtonPanel = new JPanel();
      newButtonPanel = new JPanel();
      exitButtonPanel = new JPanel();
      openProjPanel = new JPanel();
      newProjPanel = new JPanel();
      exitAppPanel = new JPanel();
      // setup Layouts
      mainPanel.setLayout(new BorderLayout());
      centerPanel.setLayout(new GridLayout(3, 2, 0, 0));
      openProjPanel.setLayout(new GridLayout(1, 2, 0, 0));
      newProjPanel.setLayout(new GridLayout(1, 2, 0, 0));
      exitAppPanel.setLayout(new GridLayout(1, 2, 0, 0));
      // setup Images, Buttons and Labels
      this.loadImages();
      this.addButtons();
      this.addLabels();

      // add listeners
      this.addWindowListener(new WindowAdapter() {
                               public void windowClosing(WindowEvent e)
                               {System.exit(0);}
                             });
      buttonListener = new OpenAppActListener();
      newProjButton.addActionListener(buttonListener);
      openProjButton.addActionListener(buttonListener);
      exitAppButton.addActionListener(buttonListener);

      // add Components to Panels
      northPanel.add(dialogLabel);
      openButtonPanel.add(openProjButton);
      newButtonPanel.add(newProjButton);
      openProjPanel.add(openButtonPanel);
      openProjPanel.add(openProjLabel);
      newProjPanel.add(newButtonPanel);
      newProjPanel.add(newProjLabel);
      exitButtonPanel.add(exitAppButton);
      exitAppPanel.add(exitButtonPanel);
      exitAppPanel.add(spacer);
      centerPanel.add(newProjPanel);
      centerPanel.add(openProjPanel);
      centerPanel.add(exitAppPanel);
      mainPanel.add(northPanel, "North");
      mainPanel.add(centerPanel, "Center");
      this.getContentPane().add(mainPanel);
      this.pack();
      this.setResizable(false);
      this.setLocationRelativeTo(parent);
      this.setVisible(true);
    }

    protected void loadImages() {
      newProjIcon = new ImageIcon(properties.getString("NEW_PROJ_ICON"),
                                  properties.getString("NEW_TIP"));
      openProjIcon = new ImageIcon(properties.getString("OPEN_PROJ_ICON"),
                                   properties.getString("OPEN_TIP"));
    }

    protected void addButtons() {
      newProjButton = new JButton(newProjIcon);
      newProjButton.setBorder(BorderFactory.createRaisedBevelBorder());
      openProjButton = new JButton(openProjIcon);
      openProjButton.setBorder(BorderFactory.createRaisedBevelBorder());
      exitAppButton = new JButton(properties.getString("EXIT"));
      exitAppButton.setFont(new Font("Times", Font.BOLD, 24));
      exitAppButton.setBorder(BorderFactory.createRaisedBevelBorder());
    }


    protected void addLabels() {
      dialogLabel = new JLabel(properties.getString("APP_DIALOG"));
      newProjLabel = new JLabel(properties.getString("NEW_TIP"));
      openProjLabel = new JLabel(properties.getString("OPEN_TIP"));
    }

    class OpenAppActListener implements ActionListener {
      public void actionPerformed(ActionEvent event) {
        Object object = event.getSource();
        if (object == newProjButton) {
          new_App_Action(event);
        } else if (object==openProjButton) {
          open_App_Action(event);
        } else if (object==exitAppButton) {
          exit_App_Action(event);
        }
      }
    }

    private void new_App_Action(ActionEvent event) {
      new_Action(event);
      if (prj != null)
        this.dispose();
    }

    private void open_App_Action(ActionEvent event) {
      open_Action(event);
      if (prj != null)
        this.dispose();
    }

    private void exit_App_Action(ActionEvent event) {
      int reply = JOptionPane.showConfirmDialog(this,
                                                properties.getString("EXIT_DIALOG"),
                                                properties.getString("EXIT"),
                                                JOptionPane.YES_NO_OPTION,
                                                JOptionPane.QUESTION_MESSAGE);
      if (reply==JOptionPane.NO_OPTION) return;
      // User answered "Yes", so  exit
      //
      exit_Action(event);
    }
  }

  class ModelCreationFrame extends JDialog {

    // panels
    JPanel northPanel;
    JPanel centerPanel;
    JPanel mainPanel;
    JPanel simplePanel;
    JPanel conceptPanel;
    JPanel evidencePanel;
    JPanel cancelPanel;

    // icons
    ImageIcon simpleIcon;
    ImageIcon conceptIcon;
    ImageIcon evidenceIcon;

    // buttons
    JButton simpleButton;
    JButton conceptButton;
    JButton evidenceButton;
    JButton cancelButton;

    // labels
    JLabel dialogLabel;
    JLabel simpleLabel;
    JLabel conceptLabel;
    JLabel evidenceLabel;
    JLabel spacer = new JLabel("");

    public ModelCreationFrame(JFrame parent, String title) {
      super(parent, title, true);
      this.setVisible(false);
      CreateModelActListener modListener = new CreateModelActListener();
      northPanel = new JPanel();
      centerPanel = new JPanel();
      mainPanel = new JPanel();
      simplePanel = new JPanel();
      conceptPanel = new JPanel();
      evidencePanel = new JPanel();
      cancelPanel = new JPanel();

      mainPanel.setLayout(new BorderLayout());
      centerPanel.setLayout(new GridLayout(4, 2, 0, 0));

      loadImages();
      addLabels();
      addButtons();

      northPanel.add(dialogLabel);

      simpleButton.addActionListener(modListener);
      conceptButton.addActionListener(modListener);
      evidenceButton.addActionListener(modListener);
      cancelButton.addActionListener(modListener);

      simplePanel.add(simpleButton);
      conceptPanel.add(conceptButton);
      evidencePanel.add(evidenceButton);
      cancelPanel.add(cancelButton);

      centerPanel.add(simplePanel);
      centerPanel.add(simpleLabel);
      centerPanel.add(conceptPanel);
      centerPanel.add(conceptLabel);
      centerPanel.add(evidencePanel);
      centerPanel.add(evidenceLabel);
      centerPanel.add(cancelPanel);
      centerPanel.add(spacer);

      mainPanel.add(northPanel, "North");
      mainPanel.add(centerPanel, "Center");
      this.getContentPane().add(mainPanel);
      this.pack();
      this.setResizable(false);
      this.setLocationRelativeTo(parent);
      this.setVisible(true);
    }


    public void loadImages() {
      simpleIcon = new ImageIcon(properties.getString("SIMPLE_MAP_ICON"),
                                 properties.getString("CREATE_SIMPLE_MODEL"));
      conceptIcon = new ImageIcon(properties.getString("CONCEPT_MAP_ICON"),
                                  properties.getString("CREATE_CONCEPT_MODEL"));
      evidenceIcon = new ImageIcon(properties.getString("EVIDENCE_MAP_ICON"),
                                  properties.getString("CREATE_EVIDENCE_MODEL"));
    }


    public void addLabels() {
      dialogLabel = new JLabel(properties.getString("MODEL_CREATE_DIALOG"));
      simpleLabel = new JLabel(properties.getString("SIMPLE_MODEL"));
      conceptLabel = new JLabel(properties.getString("CONCEPT_MODEL"));
      evidenceLabel = new JLabel(properties.getString("EVIDENCE_MODEL"));
    }


    public void addButtons() {
      simpleButton = new JButton(simpleIcon);
      simpleButton.setActionCommand("belvedere.model.Model");
      simpleButton.setBorder(BorderFactory.createRaisedBevelBorder());
      conceptButton = new JButton(conceptIcon);
      conceptButton.setActionCommand("belvedere.model.concept.ConceptualModel");
      conceptButton.setBorder(BorderFactory.createRaisedBevelBorder());
      evidenceButton = new JButton(evidenceIcon);
      evidenceButton.setActionCommand("belvedere.model.evidence.EvidenceModel");
      evidenceButton.setBorder(BorderFactory.createRaisedBevelBorder());
      cancelButton = new JButton(properties.getString("CANCEL"));
      cancelButton.setFont(new Font("Times", Font.BOLD, 20));
      cancelButton.setBorder(BorderFactory.createRaisedBevelBorder());
    }


    class CreateModelActListener implements ActionListener {
      public void actionPerformed(ActionEvent event) {
        Object object = event.getSource();
        if (object == cancelButton) {
          close_Action(event);
        } else {
          create_Action(event);
        }
      }
    }

    public void create_Action(ActionEvent event) {
      addModel_Action(event);
      this.dispose();
    }

    public void close_Action(ActionEvent event) {
      this.dispose();
    }
  }

  public void modelRemoved(Project p, Model m, int index) {
    pnlTop.removeTabAt(index);
    pnlBottom.removeTabAt(index);
  }


  public void modelAdded(Project p, Model m) {
    ModelPanel mpTop = new belvedere.ModelPanel(m,this);
    ModelPanel mpBottom = new belvedere.ModelPanel(m,this);
    m.addModelChangeListener(this);
    pnlTop.addTab(m.getName(),mpTop);
    pnlBottom.addTab(m.getName(),mpBottom);
    pnlTop.setSelectedIndex(pnlTop.getTabCount()-1);
    pnlBottom.setSelectedIndex(pnlBottom.getTabCount()-1);
  }

  /**
   * fired when the model is renamed, comes from the ModelChangeListener
   *
   * @param m      model involved in the renaming
   */
  public void modelRenamed(Model m) {
    int index = 0;
    for (Iterator i=prj.getModels().iterator();i.hasNext();index++) {
      if (i.next()==m) {
        pnlTop.setTitleAt(index,m.getName());
        pnlBottom.setTitleAt(index,m.getName());
        break;
      }
    }
  }

  // ModelChangeListener events not being used
  public void constituentDelete(Model m, Constituent c) {}
  public void constituentInsert(Model m, Constituent c) {}
  public void constituentSetText(Model m, Constituent c) {}
  public void constituentObjectReplace(Model m, Constituent oldCons, Constituent newCons) {}
  public void constituentSelect(Model m, Constituent c) {}

}
