/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere.graphview;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Point;
import javax.swing.JFrame;

import belvedere.model.*;

public interface VisualConstituent {

    /**
     * sets the border color for a visual constituent when it is not selected
     * or hovered
     *
     * @param c      the color to set the border to
     */
    public void setBorderColor(Color c);

    /**
     * sets the border color for a visual constituent when it is selected
     * 
     * @param c      the color to set the border to
     */
    public void setSelBorderColor(Color c);

    /**
     * sets the border stroke width for a visual constituent when not selected
     * or hovered
     *
     * @param n      the stroke width to set the border to
     */
    public void setBorderStrokeStrength(int n);

    /**
     * sets the border color for a visual constituent when it is hovered
     * 
     * @param c      the color to set the border to
     */
    public void setHoverBorderColor(Color c);

    /**
     * this method is called to make the constituent switch it appearance
     *  properties to a selected state
     */
    public void select();

    /**
     * this method is called to make the constituent switch its appearance
     *  properties to an unselected/not hovered state
     */
    public void deselect();

    /**
     * this method is called to make the constituent switch it appearance
     *  properties to a hovered state
     */
    public void hover();

    /**
     * from this method a constituent will display its property editor
     * 
     * @param parent the parent component that a dialog may use to center over
     */
    public void showPropEditor(JFrame parent, Model mdl);

    /**
     * used to paint the visual constituent on the given graphics
     * 
     * @param g      graphics to paint upon
     */
    public void paint(Graphics g);

    /**
     * determines whether the given point is within the Constituents bounds
     * 
     * @param x      x coordinate to test
     * @param y      y coordinate to test
     * @return true if x,y is withinBounds, false otherwise
     */
    public boolean withinBounds(int x, int y);

    /**
     * returns the horizontal center of the constituent
     * 
     * @return horizontal center of the constituent
     */
    public int getCenterX();

    /**
     * returns the vertical center of the constituent
     * 
     * @return vertical center of the constituent
     */    
    public int getCenterY();

}
