/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere.io;

import java.awt.Frame;
import java.awt.FileDialog;
import java.io.IOException;
import java.io.File;

import java.util.List;
import java.util.Iterator;
import org.jdom.*;
import org.jdom.input.SAXBuilder;
import belvedere.model.concept.*;
import belvedere.model.Constituent;
import belvedere.model.Model;
import javax.swing.JOptionPane;


public class PicoMapImporter {

  private Frame parent;
  private ConceptualModel picoModel;
  private org.jdom.Document picoDoc;
  private Element root;

  public PicoMapImporter(Frame parent) {
    this.parent = parent;
  }


  public Model importModel() {
    return this.picoModel;
  }

  public void openPicoMapFile() {

    FileDialog openFileDialog = new FileDialog(this.parent, "Open PicoMap XML File",
                                               FileDialog.LOAD);
    openFileDialog.setModal(true);
    openFileDialog.setVisible(true);
    if (openFileDialog.getFile() != null) {
      try {
        File picoFile = new File(openFileDialog.getDirectory() + openFileDialog.getFile());
        parsePicoMapFile(picoFile);
      } catch (JDOMException je) {
        JOptionPane.showMessageDialog(this.parent, je.getMessage(),
                                      "Bad File", JOptionPane.ERROR_MESSAGE);
      } catch (IOException ioe) {
        JOptionPane.showMessageDialog(this.parent, "Unable to open file " +
                                      ioe.getMessage(), "Error opening File",
                                      JOptionPane.ERROR_MESSAGE);

      } catch (NullPointerException npe) {
        JOptionPane.showMessageDialog(this.parent, "Unable to parse file " +
                                      npe.getMessage(), "Error parsing File",
                                      JOptionPane.ERROR_MESSAGE);
        npe.printStackTrace();
      }
    }
  }


  private void parsePicoMapFile(File picoFile)
  throws IOException, JDOMException {
    SAXBuilder builder = new SAXBuilder("org.apache.xerces.parsers.SAXParser");
    this.picoDoc = builder.build(picoFile);
    this.root = picoDoc.getRootElement();
    if (!((root.getName().toLowerCase()).equals("picobel")))
      throw new JDOMException("Not a valid picomap xml file.");
    this.constructModel();
  }


  private void constructModel() throws JDOMException {
    Element model = this.root.getChild("model");
    if (model == null)
      throw new JDOMException("No valid model.");
    String name = model.getAttributeValue("name");
    String creator = model.getAttributeValue("creator");
    this.picoModel = new ConceptualModel(name);
    List nodes = model.getChildren("Node");
    ConceptualNode picoNode;
    Constituent[] constituents = new Constituent[nodes.size()];
    String text;
    int posX, posY;
    int nodeId;
    for (Iterator i = nodes.iterator(); i.hasNext();) {
      Element node = (Element)i.next();
      text = node.getAttributeValue("text");
      nodeId = node.getAttribute("id").getIntValue();
      posX = 3 * node.getAttribute("x").getIntValue();
      posY = 3 * node.getAttribute("y").getIntValue();
      picoNode = new ConceptualNode(text, null, null, creator, null, 3, posX,
                                    posY, 200);
      picoModel.insertVertex(picoNode);
      constituents[nodeId] = picoNode;
    }
    ConceptualRelation picoRelation;
    int id, sourceId, destId;
    List relations = model.getChildren("Relation");
    for (Iterator i = relations.iterator(); i.hasNext();) {
      Element relation = (Element)i.next();
      text = relation.getAttributeValue("text");
      sourceId = relation.getAttribute("sourceId").getIntValue();
      destId = relation.getAttribute("targetId").getIntValue();
      picoRelation = new ConceptualRelation(text, null, null, creator, null, 3);
      picoRelation.setOrigin(constituents[sourceId]);
      picoRelation.setDestination(constituents[destId]);
      picoModel.insertEdge(picoRelation);
    }
  }

}


