/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere.model;

/**
 * interface that allows the listeners to be notified when the model has
 * changed
 */
public interface ProjectChangeListener {

  /**
   * fired when a model is removed
   * 
   * @param p      Project involved in the model remove
   * @param m      model being removed
   * @param index  index in ArrayList of model being removed, this can be used by objects
   *               keeping track of the models in the same order as they are in the ArrayList
   */
  public void modelRemoved(Project p, Model m, int index);

/**
 * fired when a model is added
 * 
 * @param p      Project involved in the model add
 * @param m      model being added
 */
  public void modelAdded(Project p, Model m);

}
