/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere.model;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.border.*;

import belvedere.model.*;

/**
 * The basic property editor.  Models that wish to have more advanced property
 * editors should extend this editor and add their own panels.  Look at
 * EvidencePropertyEditor.java for an example on how to do this
 */
public class ProjectModelPropertyEditor extends JDialog implements ActionListener {

  protected JTextField txtName, txtCreator, txtCreationDate;
  protected JTextArea txtDescription;
  protected JButton btnOK, btnCancel;
  protected JPanel pnlCenter;

  protected boolean modalResult = false;

  protected Object propObject;

  /**
   * Constructor for PropertyEditor
   * 
   * @param title  the title to display in the window.
   */
  public ProjectModelPropertyEditor(JFrame parent, String title, Model m) {
    super(parent,title,true);
    //this.setResizable(false);
    this.propObject = m;
    //long method that draws the UI elements
    setUpUI(title);
    this.txtName.setText(m.getName());
    this.txtCreator.setText(m.getCreator());
    this.txtCreationDate.setText(m.getCreationDate());
    this.txtDescription.setText(m.getDescription());
    pack();
    setResizable(false);
    setLocationRelativeTo(parent);
  }

  /**
   * Constructor for PropertyEditor
   * 
   * @param title  the title to display in the window.
   */
  public ProjectModelPropertyEditor(JFrame parent, String title, Project p) {
    super(parent,title,true);
    //this.setResizable(false);
    this.propObject = p;
    //long method that draws the UI elements
    setUpUI(title);
    this.txtName.setText(p.getName());
    this.txtCreator.setText(p.getCreator());
    this.txtCreationDate.setText(p.getCreationDate());
    this.txtDescription.setText(p.getDescription());
    pack();
    setResizable(false);
    setLocationRelativeTo(parent);
  }

  /**
   * Used to display the property editor
   * 
   * @return true if the Constituent was changed, false otherwise
   */
  public boolean showEditor() {
    show();
    if (this.modalResult) {
      if (this.propObject instanceof Project) {
        Project p = (Project)this.propObject;
        p.setName(txtName.getText());
        p.setCreator(txtCreator.getText());
        p.setCreationDate(txtCreationDate.getText());
        p.setDescription(txtDescription.getText());
      } else {
        Model m = (Model)this.propObject;
        m.setName(txtName.getText());
        m.setCreator(txtCreator.getText());
        m.setCreationDate(txtCreationDate.getText());
        m.setDescription(txtDescription.getText());
      }

    }
    return modalResult;
  }

  /**
   * The UI event method catcher for this UI.
   * 
   * @param evt    the event object sent for a UI event.
   */
  public void actionPerformed(ActionEvent evt) {
    Object src = evt.getSource();
    if (src==btnOK) {
      this.modalResult = true;
    }
    setVisible(false);
  }

  /**
   * This long method draws the UI elements.
   */
  private void setUpUI(String borderString) {

    this.pnlCenter = new JPanel(new GridLayout(0,1));
    getContentPane().add(BorderLayout.CENTER,pnlCenter);

    GridBagLayout gb = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    JPanel pnlBasic = new JPanel(gb);

    pnlBasic.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),"Project Properties"));

    JLabel lblName = new JLabel("Name:");
    gbc.gridx = 0;
    gbc.gridy = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.NONE;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gb.setConstraints(lblName,gbc);
    pnlBasic.add(lblName);

    this.txtName = new JTextField();    
    gbc.gridx = 1;
    gbc.gridy = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.anchor = GridBagConstraints.WEST;
    gb.setConstraints(this.txtName,gbc);
    pnlBasic.add(this.txtName);

    JLabel lblCreator = new JLabel("Creator:");
    gbc.gridx = 0;
    gbc.gridy = 1;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.NONE;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gb.setConstraints(lblCreator,gbc);
    pnlBasic.add(lblCreator);

    this.txtCreator = new JTextField();
    gbc.gridx = 1;
    gbc.gridy = 1;
    gbc.gridwidth = 3;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.anchor = GridBagConstraints.WEST;
    gb.setConstraints(this.txtCreator,gbc);
    pnlBasic.add(this.txtCreator);

    JLabel lblCreationDate = new JLabel("Creation Date:");
    gbc.gridx = 0;
    gbc.gridy = 2;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.NONE;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gb.setConstraints(lblCreationDate,gbc);
    pnlBasic.add(lblCreationDate);

    this.txtCreationDate = new JTextField();
    gbc.gridx = 1;
    gbc.gridy = 2;
    gbc.gridwidth = 3;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.anchor = GridBagConstraints.WEST;
    gb.setConstraints(this.txtCreationDate,gbc);
    pnlBasic.add(this.txtCreationDate);

    JLabel lblDescription = new JLabel(borderString);
    gbc.gridx = 0;
    gbc.gridy = 3;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.NONE;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gb.setConstraints(lblDescription,gbc);
    pnlBasic.add(lblDescription);

    this.txtDescription = new JTextArea(5,40);
    this.txtDescription.setLineWrap(true);
    JScrollPane sp = new JScrollPane(this.txtDescription);
    gbc.gridx = 1;
    gbc.gridy = 3;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 50;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.anchor = GridBagConstraints.WEST;
    gb.setConstraints(sp,gbc);
    pnlBasic.add(sp);

    pnlCenter.add(pnlBasic);

    JPanel pnlSouth = new JPanel();
    this.btnOK = new JButton("OK");
    this.btnOK.addActionListener(this);
    this.btnCancel = new JButton("Cancel");
    this.btnCancel.addActionListener(this);
    pnlSouth.add(btnOK);
    pnlSouth.add(btnCancel);
    getContentPane().add(BorderLayout.SOUTH,pnlSouth);

  }

}
