/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere.model.evidence;

import java.awt.Color;

import belvedere.model.Constituent;
import belvedere.model.Node;

/**
 * class used to make a DataNode object
 */
public class DataNode extends EvidenceNode {

  /**
   * Constructor for DateNode
   */
  public DataNode() {
    super("","","","","",Constituent.NEUTRALLY_BELIEVE);
    this.regBackColor = Color.green;
    this.backColor = this.regBackColor;
  }

  /**
   * Constructor for DateNode
   * 
   * @param text     text for the node
   * @param URL      URL for the node
   * @param notes    notes for the node
   * @param creator  creator of the node
   * @param modifier modifier of the node
   * @param strength strength of the node, indicated in line width
   */
  public DataNode(String text, String URL, String notes, String creator,
                  String modifier, int strength) {

    super(text, URL, notes, creator, modifier, strength);
    this.regBackColor = Color.green;
    this.backColor = this.regBackColor;
  }

  /**
   * Constructor for DataNode
   * 
   * @param text     text for the node
   * @param URL      URL for the node
   * @param notes    notes for the node
   * @param creator  creator of the node
   * @param modifier modifier of the node
   * @param strength strength of the node, indicated in line width
   * @param x        x coordinate of node
   * @param y        y coordinate of node
   * @param w        width of node
   */
  public DataNode(String text, String URL, String notes, String creator,
                  String modifier, int strength, int x, int y, int w) {
    super(text, URL, notes, creator, modifier, strength, x, y, w);
    this.regBackColor = Color.green;
    this.backColor = this.regBackColor;
  }


  /**
   * Returns a new <code>DataNode</code> object which is a copy of this instance.
   *
   * @return clone  a copy <code>Object</code> instance of this <code>DataNode</code>.
   */
  public Object clone() {
    return new DataNode(this.text, this.URL, this.notes, 
                        this.creator, this.modifier, this.strength, 
                        this.posX, this.posY, this.width);
  }

}
