/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere.model.evidence;

import javax.swing.JFrame;

import belvedere.model.*;

/**
 * abstract class foundation for nodes in the evidence model
 */
public abstract class EvidenceNode extends Node {

  /**
   * Constructor for EvidenceNode
   */
  public EvidenceNode() {
    super("", "", "", "", "", Constituent.NEUTRALLY_BELIEVE);
  }

  /**
   * Constructor for EvidenceNode
   * 
   * @param text     text for the node
   * @param URL      URL for the node
   * @param notes    notes for the node
   * @param creator  creator of the node
   * @param modifier modifier of the node
   * @param strength strength of the node, indicated in line width
   */
  public EvidenceNode(String text, String URL, String notes, String creator,
                      String modifier, int strength) {
    super(text, URL, notes, creator, modifier, strength);
  }

  /**
   * Constructor for EvidenceNode
   * 
   * @param text     text for the node
   * @param URL      URL for the node
   * @param notes    notes for the node
   * @param creator  creator of the node
   * @param modifier modifier of the node
   * @param strength strength of the node, indicated in line width
   * @param x        x coordinate of node
   * @param y        y coordinate of node
   * @param w        width of node
   */
  public EvidenceNode(String text, String URL, String notes, String creator,
                      String modifier, int strength, int x, int y, int w) {
    super(text, URL, notes, creator, modifier, strength, x, y, w);
  }

  /**
   * from this method a constituent will display its property editor
   * 
   * @param parent the parent component that a dialog may use to center over
   */
  public void showPropEditor(JFrame parent, Model mdl) {
    EvidenceConstituentPropertyEditor p = new EvidenceConstituentPropertyEditor(parent,"Edit Properties",this);
    p.showEditor(mdl);
  }

}
