/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere.model.evidence;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;

import belvedere.model.*;
import belvedere.graphview.VisualRelation;

/**
 * class used to make a ForRelation object
 */
public class ForRelation extends EvidenceRelation {

  /**
   * Constructor for ForRelation
   */
  public ForRelation() {
    super("","","","","",Constituent.NEUTRALLY_BELIEVE);
    this.regBorderColor = Color.green;
    //set up current as regular
    this.borderColor = this.regBorderColor;
  }

  /**
   * Constructor for ForRelation
   * 
   * @param text     text for the relation
   * @param URL      URL for the relation
   * @param notes    notes for the relation
   * @param creator  creator of the relation
   * @param modifier modifier of the relation
   * @param strength strength of the relation, indicated in line width
   */
  public ForRelation(String text, String URL, String notes, 
                     String creator, String modifier, int strength) {
    super(text,URL,notes,creator,modifier,strength);
    this.regBorderColor = Color.green;
    //set up current as regular
    this.borderColor = this.regBorderColor;
  }


  /**
   * Returns a new <code>ForRelation</code> object which is a copy of this instance.
   *
   * @return clone  a copy <code>Object</code> instance of this 
   *               <code>ForRelation</code>.
   */
  public Object clone() {
    return new ForRelation(this.text, this.URL, this.notes, this.creator, 
                           this.modifier, this.strength);
  }

  /**
   * The function to paint the line
   * 
   * @param g      the graphics to paint the line to
   */
  public void paint(Graphics g) {
    super.paint(g, "+");
  }

  /**
   * The function to paint the line when only one end is anchored, for example
   * in the dragging state
   * 
   * @param g      the graphics to paint the line to
   * @param x2     x coordinate to paint to
   * @param y2     y coordinate to paint to
   */
  public void paint(Graphics g, int x2, int y2) {
    super.paint(g,x2,y2, "+");
  }

}
