/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere.model.evidence;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;

import belvedere.model.Constituent;
import belvedere.model.Node;

/**
 * class used to make an UnspecifiedNode object
 */
public class UnspecifiedNode extends EvidenceNode {

  private Cloud cloud;

  /**
   * Constructor for UnspecifiedNode
   */
  public UnspecifiedNode() {
    super("", "", "", "", "", Constituent.NEUTRALLY_BELIEVE);
    this.regBackColor = Color.white;
    this.backColor = this.regBackColor;
    this.cloud = new Cloud();
  }

  /**
   * Constructor for UnspecifiedNode
   * 
   * @param text     text for the node
   * @param URL      URL for the node
   * @param notes    notes for the node
   * @param creator  creator of the node
   * @param modifier modifier of the node
   * @param strength strength of the node, indicated in line width
   */
  public UnspecifiedNode(String text, String URL, String notes, String creator,
                         String modifier, int strength) {
    super(text, URL, notes, creator, modifier, strength);
    this.regBackColor = Color.white;
    this.backColor = this.regBackColor;
    this.cloud = new Cloud();
  }

  /**
   * Constructor for UnspecifiedNode
   * 
   * @param text     text for the node
   * @param URL      URL for the node
   * @param notes    notes for the node
   * @param creator  creator of the node
   * @param modifier modifier of the node
   * @param strength strength of the node, indicated in line width
   * @param x        x coordinate for node
   * @param y        y coordinate for node
   * @param w        width for node
   */
  public UnspecifiedNode(String text, String URL, String notes, String creator,
                         String modifier, int strength, int x, int y, int w) {
    super(text, URL, notes, creator, modifier, strength, x, y, w);
    this.regBackColor = Color.white;
    this.backColor = this.regBackColor;
    this.cloud = new Cloud();
  }

  /**
   * Returns a new <code>UnspecifiedNode</code> object which is a copy of this instance.
   *
   * @return clone  a copy <code>Object</code> instance of this <code>UnspecifiedNode</code>.
   */
  public Object clone() {
    return new UnspecifiedNode(this.text, this.URL, this.notes, 
                               this.creator, this.modifier, this.strength, 
                               this.posX, this.posY, this.width);
  }

  /**
   * used to paint the visual node on the given graphics
   * 
   * @param g      graphics to paint upon
   */
  public void paint(Graphics g) {
    Graphics2D g2D = (Graphics2D)g;
    g2D.setStroke(borderStroke);
    g2D.setColor(backColor);      
    this.cloud.reset();   // forget all previous coordinates
    this.cloud.setBounds(this.posX, this.posY, this.width, this.height);
    this.cloud.setBorder();
    g2D.setColor(borderColor);
    g2D.draw(cloud);
    this.cloud.fillShape();
    g2D.setColor(backColor);
    g2D.fill(cloud);
    g2D.setColor(textColor);
    g2D.setFont(textFont);
    int textPosX = posX + textInset;
    int textPosY = posY + textAscent + textInset;
    for (int i=0;i<displayText.size();i++)
      g2D.drawString((String)displayText.get(i),textPosX,textPosY + (i*textHeight));
  }

}
