/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere.util;

/**
 * This class is used to create objects that describe how to create a button.
 */
public class ButtonDescriptor {

  private String actionCommand;
  private String imageIcon;
  private String toolTipText;

  /**
   * Constructor that sets all private fields of a ButtonDescriptor
   * 
   * @param actionCommand
   *                  string that is used to set the actionCommand of a button
   * @param imageIcon string that is used as the path to the icon for the button
   * @param toolTipText
   *                  used as tooltip text for the button
   */
  public ButtonDescriptor(String actionCommand, String imageIcon, String toolTipText) {
    this.actionCommand = actionCommand;
    this.imageIcon = imageIcon;
    this.toolTipText = toolTipText;
  }

  /**
   * returns the action command for this button
   * 
   * @return action command for this button
   */
  public String getActionCommand() {
    return this.actionCommand;
  }

  /**
   * returns the path to the image for this button
   * 
   * @return path to image for this button
   */
  public String getImageIcon() {
    return this.imageIcon;
  }

  /**
   * returns the tooltip text for the button
   * 
   * @return tooltip text
   */
  public String getToolTipText() {
    return this.toolTipText;
  }
}
