/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) <year> <name of author>
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere;

import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import javax.swing.JComponent;

import belvedere.graphview.VisualNode;

/**
 * the GlassPane used in the application to facilitate drag and drop between
 * components
 */
public class BelvedereGlassPane extends JComponent implements MouseMotionListener {
  
  private Object dragObject; // the object being dragged

  /**
   * main constructor that just sets up a MouseMotionListener
   */
  public BelvedereGlassPane() {
    addMouseMotionListener(this);
  }

  /**
   * used on the initiation of a drag, sets the object associated with the drag
   * 
   * @param dragObject object associated with the drag
   */
  public void setDragObject(Object dragObject) {
    this.dragObject = dragObject;
  }

  /**
   * overrides the paint method to paint the drag object if it supports the paint
   * method
   * 
   * @param g      Graphics object that is being painted to
   */
  public void paint(Graphics g) {
    try {
      ((VisualNode)dragObject).paint(g);
    } catch (Exception e) {
      // for model dragging we will eat the exception for now that
      // gets thrown because we haven't set vn to an object that
      // is paintable
    }
  }

  /**
   * Invoked when a mouse button is pressed on a component and then
   * dragged.  Mouse drag events will continue to be delivered to
   * the component where the first originated until the mouse button is
   * released (regardless of whether the mouse location is within the
   * bounds of the component).  Therefore this GlassPane receives the events
   * via event dispatching
   * 
   * @param e      associated MouseEvent
   */
  public void mouseDragged(MouseEvent e) {
    try {
      ((VisualNode)dragObject).setX(e.getX());
      ((VisualNode)dragObject).setY(e.getY());
      repaint();
    } catch (Exception exc) {
      // for model dragging we will eat the exception for now that
      // gets thrown because we haven't set vn to an object that
      // has coordinates
    }
  }

  /**
   * Invoked when the mouse button has been moved on a component
   * (with no buttons no down).
   */
  public void mouseMoved(MouseEvent e) {};

}
