/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere.graphview;

import java.awt.Color;
import java.awt.Graphics;

/**
 * interface that all nodes used in a graph panel must implement
 */
public interface VisualNode extends VisualConstituent {

    /**
     * sets the back color for a visual node when it is not selected/hovered
     * 
     * @param c      the color to set the back to
     */    
    public void setBackColor(Color c);

    /**
     * sets the back color for a visual node when it is selected
     * 
     * @param c      the color to set the back to
     */    
    public void setSelBackColor(Color c);

    /**
     * sets the back color for a visual node when it is hovered
     * 
     * @param c      the color to set the back to
     */    
    public void setHoverBackColor(Color c);

    /**
     * sets the x coordinate for the node
     * 
     * @param x      new x coordinate for node
     */
    public void setX(int x);

    /**
     * sets the y coordinate for the node
     * 
     * @param y      new y coordinate for node
     */
    public void setY(int y);

    /**
     * returns the x coordinate of the node
     * 
     * @return x coordinate of node
     */
    public int getX();

    /**
     * returns the y coordinate of the node
     * 
     * @return y coordinate of node
     */
    public int getY();
    
    /**
     * sets the width for the node
     * 
     * @param width  new width for the node
     */
    public void setWidth(int width);

    /**
     * sets the height for the node
     * 
     * @param height new height for the node
     */
    public void setHeight(int height);

    /**
     * returns the width of the node
     * 
     * @return width of the node
     */
    public int getWidth();

    /**
     * returns the height of the node
     * 
     * @return height of node
     */
    public int getHeight();

}
