/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere.io;

import org.jdom.*;
import org.jdom.input.SAXBuilder;

import java.io.File;
import java.io.IOException;

import belvedere.io.Belvedere3Importer;
import belvedere.model.Project;

/**
 * Object used to determine the belvedere version number of a given file
 */
public class FileVersioner {

  public static final int VERSION_3 = 0;
  public static final int VERSION_4 = 1;
  public static final int INVALID_FILE = 2;

  /**
   * Opens file filename and determines the belvedere version
   * 
   * @param filename file to open
   * @return determines which version file was created with and returns appropriate constant
   */
  public static int getVersion(String filename) {
    try {
      File belFile = new File(filename);
      SAXBuilder builder = new SAXBuilder("org.apache.xerces.parsers.SAXParser");
      Document doc = builder.build(belFile);
      Element root = doc.getRootElement();
      if (root.getName().equals(Belvedere3Importer.BELVEDERE_ROOT))
        return VERSION_3;
      else if (root.getName().equals(Project.PROG_NAME_KEY))
        // PROG_NAME_KEY indicates version 4, later versions will require
        // checking the PROG_VER_KEY
        return VERSION_4;
      else
        return INVALID_FILE;
    } catch (Exception e) {
      return INVALID_FILE;
    }
  }

}
