/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere.model;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.border.*;

import belvedere.model.*;

/**
 * The basic property editor.  Models that wish to have more advanced property
 * editors should extend this editor and add their own panels.  Look at
 * EvidencePropertyEditor.java for an example on how to do this
 */
public class ConstituentPropertyEditor extends JDialog implements ActionListener {

  protected JTextArea txtText, txtNotes;
  protected JTextField txtURL, txtCreator, txtModifier;
  protected JButton btnOK, btnCancel;
  protected JPanel pnlCenter;

  protected boolean modalResult = false;

  protected Constituent c;

  /**
   * Constructor for PropertyEditor
   * 
   * @param title  the title to display in the window.
   */
  public ConstituentPropertyEditor(JFrame parent, String title, Constituent c) {
    super(parent,title,true);
    //this.setResizable(false);
    this.c = c;
    //long method that draws the UI elements
    setUpUI();
    this.txtText.setText(c.getText());
    this.txtURL.setText(c.getURL());
    this.txtNotes.setText(c.getNotes());
    this.txtCreator.setText(c.getCreator());
    this.txtModifier.setText(c.getModifier());
    pack();
    setResizable(false);
    setLocationRelativeTo(parent);
  }

  /**
   * Used to display the property editor
   * 
   */
  public void showEditor(Model mdl) {
    show();
    if (this.modalResult) {
      this.c.setText(txtText.getText());
      this.c.setURL(txtURL.getText());
      this.c.setNotes(txtNotes.getText());
      this.c.setCreator(txtCreator.getText());
      this.c.setModifier(txtModifier.getText());
      mdl.setConstituentText(this.c,this.c.getText());
    }
  }

  /**
   * The UI event method catcher for this UI.
   * 
   * @param evt    the event object sent for a UI event.
   */
  public void actionPerformed(ActionEvent evt) {
    Object src = evt.getSource();
    if (src==btnOK) {
      this.modalResult = true;
    }
    setVisible(false);
  }

  /**
   * This long method draws the UI elements.
   */
  private void setUpUI() {

    this.pnlCenter = new JPanel(new GridLayout(0,1));
    getContentPane().add(BorderLayout.CENTER,pnlCenter);

    GridBagLayout gb = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    JPanel pnlBasic = new JPanel(gb);

    pnlBasic.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),"Basic Properties"));

    JLabel lblText = new JLabel("Text:");
    gbc.gridx = 0;
    gbc.gridy = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.NONE;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gb.setConstraints(lblText,gbc);
    pnlBasic.add(lblText);

    this.txtText = new JTextArea(5,40);
    this.txtText.setLineWrap(true);
    JScrollPane sp = new JScrollPane(this.txtText);
    gbc.gridx = 1;
    gbc.gridy = 0;
    gbc.gridwidth = 3;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.anchor = GridBagConstraints.WEST;
    gb.setConstraints(sp,gbc);
    pnlBasic.add(sp);

    JLabel lblURL = new JLabel("URL:");
    gbc.gridx = 0;
    gbc.gridy = 1;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.NONE;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gb.setConstraints(lblURL,gbc);
    pnlBasic.add(lblURL);

    this.txtURL = new JTextField();
    gbc.gridx = 1;
    gbc.gridy = 1;
    gbc.gridwidth = 3;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.anchor = GridBagConstraints.WEST;
    gb.setConstraints(this.txtURL,gbc);
    pnlBasic.add(this.txtURL);


    JLabel lblNotes = new JLabel("Notes:");
    gbc.gridx = 0;
    gbc.gridy = 2;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.NONE;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gb.setConstraints(lblNotes,gbc);
    pnlBasic.add(lblNotes);

    this.txtNotes = new JTextArea(5,0);
    this.txtNotes.setLineWrap(true);
    sp = new JScrollPane(this.txtNotes);
    gbc.gridx = 1;
    gbc.gridy = 2;
    gbc.gridwidth = 3;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.anchor = GridBagConstraints.WEST;
    gb.setConstraints(sp,gbc);
    pnlBasic.add(sp);

    JLabel lblCreator = new JLabel("Creator:");
    gbc.gridx = 0;
    gbc.gridy = 3;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.NONE;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gb.setConstraints(lblCreator,gbc);
    pnlBasic.add(lblCreator);

    this.txtCreator = new JTextField();
    gbc.gridx = 1;
    gbc.gridy = 3;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 50;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.anchor = GridBagConstraints.WEST;
    gb.setConstraints(this.txtCreator,gbc);
    pnlBasic.add(this.txtCreator);

    JLabel lblModifier = new JLabel("Modifier:");
    gbc.gridx = 2;
    gbc.gridy = 3;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.NONE;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gb.setConstraints(lblModifier,gbc);
    pnlBasic.add(lblModifier);

    this.txtModifier = new JTextField();
    gbc.gridx = 3;
    gbc.gridy = 3;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 50;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.anchor = GridBagConstraints.WEST;
    gb.setConstraints(this.txtModifier,gbc);
    pnlBasic.add(this.txtModifier);


    this.pnlCenter.add(pnlBasic);

    JPanel pnlSouth = new JPanel();
    this.btnOK = new JButton("OK");
    this.btnOK.addActionListener(this);
    this.btnCancel = new JButton("Cancel");
    this.btnCancel.addActionListener(this);
    pnlSouth.add(btnOK);
    pnlSouth.add(btnCancel);
    getContentPane().add(BorderLayout.SOUTH,pnlSouth);

  }

}
