/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere.model;

import java.io.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;

import java.util.Iterator;

import belvedere.model.*;

public class ModelRemover extends JDialog implements ActionListener {

  protected JButton btnRemove, btnDone;
  protected JList lstModels;
  protected DefaultListModel lstModel;

  protected Project prj;

  public ModelRemover(JFrame parent, String title, Project prj) {
    super(parent,title,true);
    this.prj = prj;
    //long method that draws the UI elements
    setUpUI(title);
    pack();
    resize(300,200);
    setResizable(false);
    setLocationRelativeTo(parent);
  }

  public void showModelRemover() {
    show();
  }

  /**
   * The UI event method catcher for this UI.
   * 
   * @param evt    the event object sent for a UI event.
   */
  public void actionPerformed(ActionEvent evt) {
    Object src = evt.getSource();
    if (src==btnRemove) {
      int index = lstModels.getSelectedIndex();
      if (index!=-1) {        
        this.prj.removeModel(index);
        this.lstModel.remove(index);
      }
    } else {
      // btnDone
      setVisible(false);
    }
  }

  /**
   * This long method draws the UI elements.
   */
  private void setUpUI(String borderString) {

    getContentPane().setLayout(new BorderLayout());
    GridBagLayout gb = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    JPanel pnlMain = new JPanel(gb);

    lstModel = new DefaultListModel();
    Iterator i = this.prj.getModels().iterator();
    while (i.hasNext())
      lstModel.addElement(((Model)i.next()).getName());
    this.lstModels = new JList(lstModel);
    this.lstModels.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

    JScrollPane sp = new JScrollPane(lstModels);
    gbc.gridx = 0;
    gbc.gridy = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 5;
    gbc.weightx = 1;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gb.setConstraints(sp,gbc);
    pnlMain.add(sp);    
    
    this.btnRemove = new JButton("Remove");
    this.btnRemove.addActionListener(this);
    gbc.gridx = 1;
    gbc.gridy = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.anchor = GridBagConstraints.CENTER;
    gb.setConstraints(this.btnRemove,gbc);
    pnlMain.add(this.btnRemove);

    this.btnDone = new JButton("Done");
    this.btnDone.addActionListener(this);
    gbc.gridx = 1;
    gbc.gridy = 1;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 0;
    gbc.weighty = 0;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.anchor = GridBagConstraints.CENTER;
    gb.setConstraints(this.btnDone,gbc);
    pnlMain.add(this.btnDone);
    getContentPane().add(pnlMain,BorderLayout.CENTER);

  }

}
