/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere.model.evidence;

import javax.swing.*;
import java.awt.*;
import java.lang.reflect.*;

import belvedere.model.*;
import belvedere.util.ObjectFactory;

/**
 * The property editor used for the elements of an evidence model
 */
public class EvidenceConstituentPropertyEditor extends ConstituentPropertyEditor {

  ButtonGroup bgStrength;
  ButtonGroup bgClass;

  /**
   * Constructor for EvidencePropertyEditor
   * 
   * @param parent parent frame of this dialog
   * @param title  title to put in the dialog
   * @param c      constituent which the dialog will operate on
   */
  public EvidenceConstituentPropertyEditor(JFrame parent, String title, Constituent c) {
    super(parent,title,c);
    setUpUI();
    pack();
    setLocationRelativeTo(parent);
  }

  /**
   * sets up the user interface by adding a panel to the basic property editor that
   * has the controls specific to evidence models
   */
  private void setUpUI() {
    JPanel pnlEvidenceProps = new JPanel(new GridLayout(0,2));
    JPanel pnlStrength = new JPanel(new GridLayout(0,1));
    pnlStrength.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),"Strength"));
    JPanel pnlClass = new JPanel(new GridLayout(0,1));
    pnlClass.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),"Type"));

    this.bgStrength = new ButtonGroup();

    JRadioButton rb = new JRadioButton("Strongly Believe");
    rb.setActionCommand(Constituent.STRONGLY_BELIEVE+"");
    bgStrength.add(rb);
    if (c.getStrength()==Constituent.STRONGLY_BELIEVE) rb.setSelected(true);
    pnlStrength.add(rb);

    rb = new JRadioButton("Moderately Believe");
    rb.setActionCommand(Constituent.MODERATELY_BELIEVE+"");
    this.bgStrength.add(rb);
    if (c.getStrength()==Constituent.MODERATELY_BELIEVE) rb.setSelected(true);
    pnlStrength.add(rb);

    rb = new JRadioButton("Neutrally Believe");
    rb.setActionCommand(Constituent.NEUTRALLY_BELIEVE+"");
    this.bgStrength.add(rb);
    if (c.getStrength()==Constituent.NEUTRALLY_BELIEVE) rb.setSelected(true);
    pnlStrength.add(rb);

    rb = new JRadioButton("Moderately Disbelieve");
    rb.setActionCommand(Constituent.MODERATELY_UNBELIEVE+"");
    this.bgStrength.add(rb);
    if (c.getStrength()==Constituent.MODERATELY_UNBELIEVE) rb.setSelected(true);
    pnlStrength.add(rb);

    rb = new JRadioButton("Strongly Disbelieve");
    rb.setActionCommand(Constituent.STRONGLY_UNBELIEVE+"");
    this.bgStrength.add(rb);
    if (c.getStrength()==Constituent.STRONGLY_UNBELIEVE) rb.setSelected(true);
    pnlStrength.add(rb);

    this.bgClass = new ButtonGroup();

    if (c instanceof Node) {
      rb = new JRadioButton("Data Node");
      rb.setActionCommand("class belvedere.model.evidence.DataNode");
      if (c instanceof DataNode) rb.setSelected(true);
    } else {
      rb = new JRadioButton("For Relation");
      rb.setActionCommand("class belvedere.model.evidence.ForRelation");
      if (c instanceof ForRelation) rb.setSelected(true);
    }
    this.bgClass.add(rb);
    pnlClass.add(rb);

    if (c instanceof Node) {
      rb = new JRadioButton("Hypothesis Node");
      rb.setActionCommand("class belvedere.model.evidence.HypothesisNode");
      if (c instanceof HypothesisNode) rb.setSelected(true);
    } else {
      rb = new JRadioButton("Against Relation");
      rb.setActionCommand("class belvedere.model.evidence.AgainstRelation");
      if (c instanceof AgainstRelation) rb.setSelected(true);
    }
    this.bgClass.add(rb);
    pnlClass.add(rb);

    if (c instanceof Node) {
      rb = new JRadioButton("Unspecified Node");
      rb.setActionCommand("class belvedere.model.evidence.UnspecifiedNode");
      if (c instanceof UnspecifiedNode) rb.setSelected(true);
    } else {
      rb = new JRadioButton("Unspecified Relation");
      rb.setActionCommand("class belvedere.model.evidence.UnspecifiedRelation");
      if (c instanceof UnspecifiedRelation) rb.setSelected(true);
    }
    this.bgClass.add(rb);
    pnlClass.add(rb);

    pnlEvidenceProps.add(pnlStrength);
    pnlEvidenceProps.add(pnlClass);
    pnlCenter.add(pnlEvidenceProps);

  }

  /**
   * method used to display the editor
   * 
   */
  public void showEditor(Model mdl) {
    show();
    ButtonModel bmStrength = bgStrength.getSelection();
    ButtonModel bmClass = bgClass.getSelection();
    if (modalResult) {
      c.setText(txtText.getText());
      c.setURL(txtURL.getText());
      c.setNotes(txtNotes.getText());
      c.setCreator(txtCreator.getText());
      c.setModifier(txtModifier.getText());
      c.setStrength(Integer.parseInt(bmStrength.getActionCommand()));
      mdl.setConstituentText(c,c.getText());
      if (!c.getClass().toString().equals(bmClass.getActionCommand())) {
        // sloppy here
        String className = bmClass.getActionCommand().substring(6);
        Class constituentDefinition = null;
        Class stringArg[];
        Object args[];
        if (c instanceof Node) {
          stringArg = new Class[] {String.class, String.class, String.class,
                                   String.class, String.class, int.class, int.class, int.class, int.class};
          args = new Object[] {c.getText(),c.getURL(),c.getNotes(),c.getCreator(),c.getModifier(),new Integer(c.getStrength()),
                               new Integer(((Node)c).getX()),new Integer(((Node)c).getY()),
                               new Integer(((Node)c).getWidth()) };
        } else {
          stringArg = new Class[] {String.class, String.class, String.class,
                                     String.class, String.class, int.class};
          args = new Object[] {c.getText(),c.getURL(),c.getNotes(),c.getCreator(),c.getModifier(),new Integer(c.getStrength())};
        }
        Constructor constituentConstructor;
        Object constituent;

        try {
          constituentDefinition = Class.forName(className);
          constituentConstructor = 
          constituentDefinition.getConstructor(stringArg);
          constituent = ObjectFactory.createObjectWithArgs(constituentConstructor, args);
        } catch (ClassNotFoundException e) {
          constituent = null;
        } catch (NoSuchMethodException e) {
          constituent = null;
        }
        if (constituent!=null) {          
          if (constituent instanceof Node)
            mdl.replaceVertex((Node)c,(Node)constituent);
          else {
            ((Relation)constituent).setOrigin(((Relation)c).getOrigin());
            ((Relation)constituent).setDestination(((Relation)c).getDestination());
            mdl.replaceEdge((Relation)c,(Relation)constituent);
          }
          mdl.setSelection((Constituent)constituent);
        }
      }
    }
  }

}
