/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere.model.evidence;
 
import java.util.ArrayList;

import belvedere.model.*;
import belvedere.util.ButtonDescriptor;

/**
 * A model represents a collection of nodes and relations
 * among these nodes.
 */
public class EvidenceModel extends Model {

  /**
   * Basic constructor
   */
  public EvidenceModel() {
    this.name = "Evidence Model";
  }


  /**
   * Constructor with name parameter
   * 
   * @param name
   */
  public EvidenceModel(String name) {
      this.name = name;
  }

  /**
   * this method provides an array list of ButtonDescriptors describing the
   * buttons that should be created to work with the type of Nodes that this
   * Model uses
   * 
   * @return an ArrayList of ButtonDescriptor objects
   */
  public ArrayList getNodeButtonDescriptors() {
      ArrayList al = new ArrayList();
      ButtonDescriptor bd = new ButtonDescriptor("belvedere.model.evidence.DataNode",
                                                 "../images/data.gif","Add Data Node");
      al.add(bd);
      bd = new ButtonDescriptor("belvedere.model.evidence.HypothesisNode",
                                "../images/hypo.gif","Add Hypothesis Node");
      al.add(bd);
      bd = new ButtonDescriptor("belvedere.model.evidence.UnspecifiedNode",
                                "../images/unspec.gif","Add Unspecified Node");
      al.add(bd);
      return al;
  }

  /**
   * this method provides an array list of ButtonDescriptors describing the
   * buttons that should be created to work with the type of Relations that this
   * Model uses
   * 
   * @return an ArrayList of ButtonDescriptor objects
   */
  public ArrayList getRelationButtonDescriptors() {
      ArrayList al = new ArrayList();
      ButtonDescriptor bd = new ButtonDescriptor("belvedere.model.evidence.ForRelation",
                                                 "../images/for.gif","Add For Relation");
      al.add(bd);
      bd = new ButtonDescriptor("belvedere.model.evidence.AgainstRelation",
                                "../images/against.gif","Add Against Relation");
      al.add(bd);
      bd = new ButtonDescriptor("belvedere.model.evidence.UnspecifiedRelation",
                                "../images/unknown.gif","Add Unspecified Relation");
      al.add(bd);
      return al;
  }

  /**
   * Adds all components of <code>Model</code> parameter to this 
   * <code>EvidenceModel</code> if it is of the same instance, and throws
   * an exception otherwise.
   *
   * @param model  <code>Model</code> to be combined with this <code>EvidenceModel</code>
   * @throws IncompatibleModelExceptioin if model is not an <code>EvidenceModel</code>
   */
  public void addModel(Model mergingModel) throws IncompatibleModelException {
    if (mergingModel instanceof EvidenceModel)
      super.addModel(mergingModel);
    else
      throw new IncompatibleModelException("The model being added is not an Evidence Model");
  }

}
