/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere.util;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;

/**
 * object used to load cursors
 */
public class CursorFactory {

  /**
   * creates a cursor object from the the given file
   * 
   * @param filePath cursor file to create from
   * @param name     name to assign to cursor
   * @param p        coordinates for hot spot of cursor
   * @param comp     component used to get a toolkit object from
   * @return the created cursor or null if unable to create the cursor
   */
  public static Cursor getCursor(String filePath, String name, Point p, Component comp) {
    Cursor newCursor;
    try {
      Toolkit tk = comp.getToolkit();
      Image img = tk.getImage(filePath);
      newCursor = tk.createCustomCursor(img,p,name);
    } catch (Exception e) {
      newCursor = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);
    }
    return newCursor;
  }

}
