/**
 * Belvedere: tools for collaborative knowledge mapping 
 * 
 * Copyright (C) 2001, 2002 Dan Suthers
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package belvedere.util;

import java.lang.reflect.*;

/**
 * This class provides two static functions that can be used to create objects
 * using the class name
 */
public class ObjectFactory {

  /**
   * Creates and returns an object of class className with an empty constructor
   * 
   * @param className the class of the object to create
   * @return the created object
   */
  public static Object createObject(String className) {
    Object object = null;
    try {
      Class classDefinition = Class.forName(className);
      object = classDefinition.newInstance();
    } catch (InstantiationException e) {
      object = null;
    } catch (IllegalAccessException e) {
      object = null;
    } catch (ClassNotFoundException e) {
      object = null;
    }
    return object;
  }


  /**
   * Creates and returns an object of class className with a constructor
   * 
   * @param constructor
   *                  the constructor to use
   * @param arguments the arguments to pass to the constructor
   * @return the created object
   */
  public static Object createObjectWithArgs(Constructor constructor, 
                                            Object[] arguments) {

    Object object = null;

    try {
      object = constructor.newInstance(arguments);
      return object;
    } catch (InstantiationException e) {
      object = null;
    } catch (IllegalAccessException e) {
      object = null;
    } catch (IllegalArgumentException e) {
      object = null;
    } catch (InvocationTargetException e) {
      object = null;
    }
    return object;
  }


}
